export var Action;
(function (Action) {
    Action["Added"] = "added";
    Action["Cancelled"] = "cancelled";
    Action["Deleted"] = "deleted";
    Action["Installed"] = "installed";
    Action["Updated"] = "updated";
})(Action || (Action = {}));
import Ajv from 'ajv';
export const defaultValidationErrorHandler = (message, violations) => {
    const msg = JSON.stringify({
        type: 'Typewriter JSON Schema Validation Error',
        description: `You made an analytics call (${message.event}) using Typewriter that doesn't match the ` +
            'Tracking Plan spec.',
        errors: violations,
    }, undefined, 2);
    console.warn(msg);
};
let onViolation = defaultValidationErrorHandler;
let analytics = () => {
    return window.analytics;
};
export function setTypewriterOptions(options) {
    analytics = options.analytics ? () => options.analytics || window.analytics : analytics;
    onViolation = options.onViolation || onViolation;
}
function validateAgainstSchema(message, schema) {
    const ajv = new Ajv({ allErrors: true, verbose: true });
    if (!ajv.validate(schema, message) && ajv.errors) {
        onViolation(message, ajv.errors);
    }
}
function withTypewriterContext(message = {}) {
    return {
        ...message,
        context: {
            ...(message.context || {}),
            typewriter: {
                language: 'typescript',
                version: '9.1.0',
            },
        },
    };
}
export function configSaved(props, options, callback) {
    const schema = {
        $id: 'config_saved',
        description: 'Event used by Integrations for tracking a saved config',
        properties: {
            action: {
                $id: '/properties/action',
                description: 'The action performed on the config page',
                enum: ['added', 'cancelled', 'deleted', 'installed', 'updated'],
                type: 'string',
            },
            app_key: {
                $id: '/properties/app_key',
                description: 'Key of the app that the action was taken in.',
                type: 'string',
            },
            app_name: {
                $id: '/properties/app_name',
                description: 'Name of the app that the action was taken in.',
                type: 'string',
            },
            environment_key: {
                $id: '/properties/environment_key',
                description: 'The environment in which the app is used.',
                type: 'string',
            },
            metadata: {
                $id: '/properties/metadata',
                description: 'Metadata of the event',
                type: 'object',
            },
            organization_key: {
                $id: '/properties/organization_key',
                description: 'The organization owning the space in which the app is used.',
                type: 'string',
            },
            space_key: {
                $id: '/properties/space_key',
                description: 'The space in which the app is used.',
                type: 'string',
            },
        },
        required: ['action', 'app_key', 'environment_key', 'organization_key', 'space_key'],
        type: 'object',
    };
    validateAgainstSchema(props, schema);
    const a = analytics();
    if (a) {
        a.track('config_saved', props || {}, {
            ...options,
            context: {
                ...(options?.context || {}),
                typewriter: {
                    language: 'typescript',
                    version: '',
                },
            },
        }, callback);
    }
}
export function flowStart(props, options, callback) {
    const schema = {
        $id: 'flow_start',
        description: 'Event used by Integrations for tracking the start of a flow in the AI Content Generator',
        properties: {
            ai_model: { description: 'AI model used for the action.', type: 'string' },
            ai_provider: {
                description: 'Service provider of the AI model used for the action.',
                type: 'string',
            },
            app_key: { description: 'Key of the app that the action was taken in.', type: 'string' },
            app_name: { description: 'Name of the app that the action was taken in.', type: 'string' },
            config_options: {
                description: 'Config options that have been filled out for the action.',
                properties: {
                    has_additional: { description: '', type: 'boolean' },
                    has_audience: { description: '', type: 'boolean' },
                    has_exclude: { description: '', type: 'boolean' },
                    has_include: { description: '', type: 'boolean' },
                    has_profile: { description: '', type: 'boolean' },
                    has_tone: { description: '', type: 'boolean' },
                    has_values: { description: '', type: 'boolean' },
                },
                type: 'object',
            },
            content_type_id: {
                description: 'Content type ID for the entry currently being edited.',
                type: 'string',
            },
            entry_id: { description: 'Entry ID for the entry currently being edited.', type: 'string' },
            environment_key: { description: 'The environment in which the app is used.', type: 'string' },
            feature_id: {
                description: 'The corresponding ID of the current feature. See https://github.com/contentful/apps/blob/master/apps/ai-content-generator/src/configs/features/featureConfig.tsx#L9',
                type: 'string',
            },
            organization_key: {
                description: 'The organization owning the space in which the app is used.',
                type: 'string',
            },
            space_key: { description: 'The space in which the app is used.', type: 'string' },
        },
        required: [
            'app_key',
            'environment_key',
            'organization_key',
            'space_key',
            'content_type_id',
            'entry_id',
            'ai_model',
            'ai_provider',
            'feature_id',
            'config_options',
        ],
        type: 'object',
    };
    validateAgainstSchema(props, schema);
    const a = analytics();
    if (a) {
        a.track('flow_start', props || {}, {
            ...options,
            context: {
                ...(options?.context || {}),
                typewriter: {
                    language: 'typescript',
                    version: '',
                },
            },
        }, callback);
    }
}
export function generateClicked(props, options, callback) {
    const schema = {
        $id: 'generate_clicked',
        description: 'Event used by Integrations for tracking clicks on the generate button',
        properties: {
            ai_model: { description: 'AI model used for the action.', type: 'string' },
            ai_provider: {
                description: 'Service provider of the AI model used for the action.',
                type: 'string',
            },
            app_key: { description: 'Key of the app that the action was taken in.', type: 'string' },
            app_name: { description: 'Name of the app that the action was taken in.', type: 'string' },
            config_options: {
                description: 'Config options that have been filled out for the action.',
                properties: {
                    has_additional: { description: '', type: 'boolean' },
                    has_audience: { description: '', type: 'boolean' },
                    has_exclude: { description: '', type: 'boolean' },
                    has_include: { description: '', type: 'boolean' },
                    has_profile: { description: '', type: 'boolean' },
                    has_tone: { description: '', type: 'boolean' },
                    has_values: { description: '', type: 'boolean' },
                },
                type: 'object',
            },
            content_generation_prompt: {
                description: 'The text of the prompt that was submitted. This will not include any text from a source field.',
                type: 'string',
            },
            content_type_id: {
                description: 'Content type ID for the entry currently being edited.',
                type: 'string',
            },
            entry_id: { description: 'Entry ID for the entry currently being edited.', type: 'string' },
            environment_key: { description: 'The environment in which the app is used.', type: 'string' },
            feature_id: {
                description: 'The corresponding ID of the current feature. See https://github.com/contentful/apps/blob/master/apps/ai-content-generator/src/configs/features/featureConfig.tsx#L9',
                type: 'string',
            },
            from_prompt: {
                description: 'Whether the action was performed using a prompt instead of a source.',
                type: 'boolean',
            },
            organization_key: {
                description: 'The organization owning the space in which the app is used.',
                type: 'string',
            },
            source_field: { description: 'Name of source field.', type: 'string' },
            space_key: { description: 'The space in which the app is used.', type: 'string' },
            target_locale: { description: 'Target locale of the action.', type: 'string' },
        },
        required: [
            'app_key',
            'environment_key',
            'organization_key',
            'space_key',
            'content_type_id',
            'entry_id',
            'ai_model',
            'ai_provider',
            'feature_id',
            'config_options',
        ],
        type: 'object',
    };
    validateAgainstSchema(props, schema);
    const a = analytics();
    if (a) {
        a.track('generate_clicked', props || {}, {
            ...options,
            context: {
                ...(options?.context || {}),
                typewriter: {
                    language: 'typescript',
                    version: '',
                },
            },
        }, callback);
    }
}
export function generatedContentEdited(props, options, callback) {
    const schema = {
        $id: 'generated_content_edited',
        description: 'Event used by Integrations for tracking when the user edits the generated content',
        properties: {
            ai_model: { description: 'AI model used for the action.', type: 'string' },
            ai_provider: {
                description: 'Service provider of the AI model used for the action.',
                type: 'string',
            },
            app_key: { description: 'Key of the app that the action was taken in.', type: 'string' },
            app_name: { description: 'Name of the app that the action was taken in.', type: 'string' },
            config_options: {
                description: 'Config options that have been filled out for the action.',
                properties: {
                    has_additional: { description: '', type: 'boolean' },
                    has_audience: { description: '', type: 'boolean' },
                    has_exclude: { description: '', type: 'boolean' },
                    has_include: { description: '', type: 'boolean' },
                    has_profile: { description: '', type: 'boolean' },
                    has_tone: { description: '', type: 'boolean' },
                    has_values: { description: '', type: 'boolean' },
                },
                type: 'object',
            },
            content_generation_prompt: {
                description: 'The text of the prompt that was submitted. This will not include any text from a source field.',
                type: 'string',
            },
            content_type_id: {
                description: 'Content type ID for the entry currently being edited.',
                type: 'string',
            },
            entry_id: { description: 'Entry ID for the entry currently being edited.', type: 'string' },
            environment_key: { description: 'The environment in which the app is used.', type: 'string' },
            feature_id: {
                description: 'The corresponding ID of the current feature. See https://github.com/contentful/apps/blob/master/apps/ai-content-generator/src/configs/features/featureConfig.tsx#L9',
                type: 'string',
            },
            from_prompt: {
                description: 'Whether the action was performed using a prompt instead of a source.',
                type: 'boolean',
            },
            organization_key: {
                description: 'The organization owning the space in which the app is used.',
                type: 'string',
            },
            source_locale: { description: 'Source locale of the action.', type: 'string' },
            space_key: { description: 'The space in which the app is used.', type: 'string' },
            target_locale: { description: 'Target locale of the action.', type: 'string' },
        },
        required: [
            'app_key',
            'environment_key',
            'organization_key',
            'space_key',
            'content_type_id',
            'entry_id',
            'ai_model',
            'ai_provider',
            'feature_id',
            'config_options',
        ],
        type: 'object',
    };
    validateAgainstSchema(props, schema);
    const a = analytics();
    if (a) {
        a.track('generated_content_edited', props || {}, {
            ...options,
            context: {
                ...(options?.context || {}),
                typewriter: {
                    language: 'typescript',
                    version: '',
                },
            },
        }, callback);
    }
}
export function regenerationClicked(props, options, callback) {
    const schema = {
        $id: 'regeneration_clicked',
        description: 'Event used by Integrations for tracking the regeneration of content',
        properties: {
            ai_model: { description: 'AI model used for the action.', type: 'string' },
            ai_provider: {
                description: 'Service provider of the AI model used for the action.',
                type: 'string',
            },
            app_key: { description: 'Key of the app that the action was taken in.', type: 'string' },
            app_name: { description: 'Name of the app that the action was taken in.', type: 'string' },
            config_options: {
                description: 'Config options that have been filled out for the action.',
                properties: {
                    has_additional: { description: '', type: 'boolean' },
                    has_audience: { description: '', type: 'boolean' },
                    has_exclude: { description: '', type: 'boolean' },
                    has_include: { description: '', type: 'boolean' },
                    has_profile: { description: '', type: 'boolean' },
                    has_tone: { description: '', type: 'boolean' },
                    has_values: { description: '', type: 'boolean' },
                },
                type: 'object',
            },
            content_generation_prompt: {
                description: 'The text of the prompt that was submitted. This will not include any text from a source field.',
                type: 'string',
            },
            content_type_id: {
                description: 'Content type ID for the entry currently being edited.',
                type: 'string',
            },
            entry_id: { description: 'Entry ID for the entry currently being edited.', type: 'string' },
            environment_key: { description: 'The environment in which the app is used.', type: 'string' },
            feature_id: {
                description: 'The corresponding ID of the current feature. See https://github.com/contentful/apps/blob/master/apps/ai-content-generator/src/configs/features/featureConfig.tsx#L9',
                type: 'string',
            },
            from_prompt: {
                description: 'Whether the action was performed using a prompt instead of a source.',
                type: 'boolean',
            },
            organization_key: {
                description: 'The organization owning the space in which the app is used.',
                type: 'string',
            },
            source_field: { description: 'Name of source field.', type: 'string' },
            space_key: { description: 'The space in which the app is used.', type: 'string' },
            target_locale: { description: 'Target locale of the action.', type: 'string' },
        },
        required: [
            'app_key',
            'environment_key',
            'organization_key',
            'space_key',
            'content_type_id',
            'entry_id',
            'ai_model',
            'ai_provider',
        ],
        type: 'object',
    };
    validateAgainstSchema(props, schema);
    const a = analytics();
    if (a) {
        a.track('regeneration_clicked', props || {}, {
            ...options,
            context: {
                ...(options?.context || {}),
                typewriter: {
                    language: 'typescript',
                    version: '',
                },
            },
        }, callback);
    }
}
export function sidebarRendered(props, options, callback) {
    const schema = {
        $id: 'sidebar_rendered',
        description: 'Event used by Integrations for tracking when the sidebar renders',
        properties: {
            ai_model: { description: 'AI model used for the action.', type: 'string' },
            ai_provider: {
                description: 'Service provider of the AI model used for the action.',
                type: 'string',
            },
            app_key: { description: 'Key of the app that the action was taken in.', type: 'string' },
            app_name: { description: 'Name of the app that the action was taken in.', type: 'string' },
            config_options: {
                description: 'Config options that have been filled out for the action.',
                properties: {
                    has_additional: { description: '', type: 'boolean' },
                    has_audience: { description: '', type: 'boolean' },
                    has_exclude: { description: '', type: 'boolean' },
                    has_include: { description: '', type: 'boolean' },
                    has_profile: { description: '', type: 'boolean' },
                    has_tone: { description: '', type: 'boolean' },
                    has_values: { description: '', type: 'boolean' },
                },
                type: 'object',
            },
            content_type_id: {
                description: 'Content type ID for the entry currently being edited.',
                type: 'string',
            },
            entry_id: { description: 'Entry ID for the entry currently being edited.', type: 'string' },
            environment_key: { description: 'The environment in which the app is used.', type: 'string' },
            organization_key: {
                description: 'The organization owning the space in which the app is used.',
                type: 'string',
            },
            space_key: { description: 'The space in which the app is used.', type: 'string' },
        },
        required: [
            'app_key',
            'environment_key',
            'organization_key',
            'space_key',
            'content_type_id',
            'entry_id',
            'ai_model',
            'ai_provider',
            'config_options',
        ],
        type: 'object',
    };
    validateAgainstSchema(props, schema);
    const a = analytics();
    if (a) {
        a.track('sidebar_rendered', props || {}, {
            ...options,
            context: {
                ...(options?.context || {}),
                typewriter: {
                    language: 'typescript',
                    version: '',
                },
            },
        }, callback);
    }
}
const clientAPI = {
    setTypewriterOptions,
    configSaved,
    flowStart,
    generateClicked,
    generatedContentEdited,
    regenerationClicked,
    sidebarRendered,
};
export default new Proxy(clientAPI, {
    get(target, method) {
        if (typeof method === 'string' && Object.prototype.hasOwnProperty.call(target, method)) {
            return target[method];
        }
        return () => {
            console.warn(`⚠️  You made an analytics call (${String(method)}) that can't be found. Either:
         a) Re-generate your typewriter client: \`npx typewriter\`
         b) Add it to your Tracking Plan: https://app.segment.com/segment-oscb/protocols/tracking-plans/rs_1zTHJU9fd5mt7cndWnd4PgJbMCE`);
            const a = analytics();
            if (a) {
                a.track('Unknown Analytics Call Fired', {
                    method,
                }, withTypewriterContext());
            }
        };
    },
});
